@@ -1,5 +1,3 @@
-#.onLoad <- function(library, section) {require("methods")}
-
 #setClassUnion("OptionalPOSIXct",   c("POSIXct",   "NULL"))
 # bug work around
 setClassUnion("OptionalPOSIXct",   c("POSIXct",   "logical"))
@@ -21,9 +19,12 @@ setClass("TSdb", representation( dbname="character",
 setClass("TSid",  contains="TSdb", representation(serIDs="character", 
                      conType="character",DateStamp="OptionalPOSIXct")) 
 
-# this has serIDs and source db (in TDid) as well as documentation
+# TSmeta has serIDs and source db (in TSid) as well as documentation
+setClassUnion("OptionalChar",   c("character",   "logical"))
+#  NA is logical so this allows NA
 setClass("TSmeta", contains="TSid", 
-  representation(TSdescription="character", TSdoc="character", TSlabel="character" )) 
+  representation(TSdescription="OptionalChar",    TSdoc="OptionalChar",
+                       TSlabel="OptionalChar", TSsource="OptionalChar")) 
 
 # this is a logical but has TSid (so could be used to retrieve data)
 setClass("logicalId", contains="logical",
@@ -42,8 +43,9 @@ setMethod("TSmeta",   signature(x="character", con="ANY"),
                hasVintages=con@hasVintages, hasPanels=con@hasPanels,
                conType=class(con), DateStamp=NA, 
 	       TSdoc=TSdoc(x, con=con, ...),
+               TSdescription=TSdescription(x, con=con, ...),
 	       TSlabel=TSlabel(x, con=con, ...),
-               TSdescription=TSdescription(x, con=con, ...))
+	       TSsource=TSsource(x, con=con, ...))
 	} )
 
 setMethod("TSmeta",   signature(x="character", con="missing"),
@@ -56,16 +58,19 @@ setMethod("TSmeta",   signature(x="ANY", con="missing"),
       m <- attr(x, "TSmeta")
       if(is.null(m)) new("TSmeta",serIDs=seriesNames(x), 
 	         dbname="", hasVintages=FALSE, hasPanels=FALSE,
-	         conType="", DateStamp=NA, #NULL,
-		 TSdescription= as(NA, "character"), 
-		 TSdoc =  as(NA, "character"), 
-		 TSlabel= as(NA, "character")) else m
+	         conType="", 
+		 DateStamp     = NA, 
+		 TSdescription = NA, 
+		 TSdoc         = NA,  
+		 TSlabel       = NA,
+		 TSsource      = NA) else m
       })
 
 setGeneric("TSmeta<-", 
    def= function(x, value) standardGeneric("TSmeta<-"),
    useAsDefault= function(x, value){
-      if (!is(value,"TSmeta")) stop("trying to set attribute TSmeta incorrectly.") 
+      if (!is(value,"TSmeta")) 
+         stop("trying to set attribute TSmeta incorrectly.") 
       attr(x, "TSmeta") <- value
       x
       })
@@ -92,47 +97,7 @@ setMethod("TSconnect",   signature(drv="character", dbname="character"),
    definition=function(drv, dbname, ...)
              TSconnect(dbDriver(drv), dbname=dbname, ...))
 
-setGeneric("TSdescription<-", 
-   def= function(x, value) standardGeneric("TSdescription<-"),
-   useAsDefault=  function (x, value){
-    m <- TSmeta(x)
-    m@TSdescription <- value
-    TSmeta(x) <- m
-    x})
-
-setGeneric("TSdescription", def= function(x, con=getOption("TSconnection"), ...)
-    standardGeneric("TSdescription"))
-
-setMethod("TSdescription",   signature(x="character", con="missing"),
-   definition= function(x, con=getOption("TSconnection"), ...) 
-       TSdescription(x, con=getOption("TSconnection"), ...) )
-
-setMethod("TSdescription",   signature(x="ANY", con="missing"),
-   definition=  function(x, con, ...)TSmeta(x)@TSdescription)
-
-#  next is for case where there is no method for con  
-setMethod("TSdescription",   signature(x="character", con="ANY"),
-   definition= function(x, con=getOption("TSconnection"), ...) {
-       if(is.null(con)) stop("NULL con is not allowed. See ?TSdescription.")
-       else stop("con class ", class(con), 
-       " is not supported. (Check this is a TSdbi connection, not a raw DBI connection.)")} )
-
-#  Next two methods are for case where the user mistakenly specifies
-#     serIDS="whatever"  rather than x="whatever"
-#   ( A natural mistaken as this is the syntax for other functions.)
-setMethod("TSdescription",   signature(x="missing", con="ANY"),
-   definition=  function(x, con, serIDs, ...) {
-        if(missing(serIDs)) stop("missing argument x, must be specified.")
-	else TSdescription(x=serIDs, con=con, ...)
-	})
-
-setMethod("TSdescription",   signature(x="missing", con="missing"),
-   definition=  function(x, serIDs, ...) {
-        if(missing(serIDs)) stop("missing argument x, must be specified.")
-	else TSdescription(x=serIDs, ...)
-	})
-
-# internal utilities to construct WHERE and table name
+########## internal utilities to construct WHERE and table name#########
 
 tbNm <- function(hasVintages, tbl, rV) {
 	if(hasVintages) tbl <- paste(tbl, rV, sep="_")
@@ -196,6 +161,49 @@ setWhere <- function(con, serIDs, realVintage, realPanel) {
    where
    }
 
+
+######### TSdescription #########
+
+setGeneric("TSdescription<-", 
+   def= function(x, value) standardGeneric("TSdescription<-"),
+   useAsDefault=  function (x, value){
+    m <- TSmeta(x)
+    m@TSdescription <- value
+    TSmeta(x) <- m
+    x})
+
+setGeneric("TSdescription", def= function(x, con=getOption("TSconnection"), ...)
+    standardGeneric("TSdescription"))
+
+setMethod("TSdescription",   signature(x="character", con="missing"),
+   definition= function(x, con=getOption("TSconnection"), ...) 
+       TSdescription(x, con=getOption("TSconnection"), ...) )
+
+setMethod("TSdescription",   signature(x="ANY", con="missing"),
+   definition=  function(x, con, ...)TSmeta(x)@TSdescription)
+
+#  next is for case where there is no method for con  
+setMethod("TSdescription",   signature(x="character", con="ANY"),
+   definition= function(x, con=getOption("TSconnection"), ...) {
+       if(is.null(con)) stop("NULL con is not allowed. See ?TSdescription.")
+       else stop("con class ", class(con), 
+       " is not supported. (Check this is a TSdbi connection, not a raw DBI connection.)")} )
+
+#  Next two methods are for case where the user mistakenly specifies
+#     serIDS="whatever"  rather than x="whatever"
+#   ( A natural mistaken as this is the syntax for other functions.)
+setMethod("TSdescription",   signature(x="missing", con="ANY"),
+   definition=  function(x, con, serIDs, ...) {
+        if(missing(serIDs)) stop("missing argument x, must be specified.")
+	else TSdescription(x=serIDs, con=con, ...)
+	})
+
+setMethod("TSdescription",   signature(x="missing", con="missing"),
+   definition=  function(x, serIDs, ...) {
+        if(missing(serIDs)) stop("missing argument x, must be specified.")
+	else TSdescription(x=serIDs, ...)
+	})
+
 TSdescriptionSQL <-  function(x=NULL, con=getOption("TSconnection"), 
        vintage=getOption("TSvintage"), panel=getOption("TSpanel"), 
        lang=getOption("TSlang")) {	    
@@ -203,10 +211,12 @@ TSdescriptionSQL <-  function(x=NULL, con=getOption("TSconnection"),
 	            "  FROM Meta ", setWhere(con, x, 
 		        realVintage(con, vintage, x),
 		        realPanel(con,panel)), ";", sep=""))[[1]]
-	    # r should already be char, but odbc converts NA to logical
-	    if(is.null(r))  as(NA, "character") else as(r, "character")
+	    # odbc converts NA to logical, but other db may return char
+	    if(is.null(r) || is.na(r)|| ("NA" == r)) NA else r
 	    }
 
+######### TSdoc #########
+
 setGeneric("TSdoc<-", 
    def= function(x, value) standardGeneric("TSdoc<-"),
    useAsDefault=  function (x, value){
@@ -215,7 +225,6 @@ setGeneric("TSdoc<-",
     TSmeta(x) <- m
     x})
 
-
 setGeneric("TSdoc", 
    def= function(x, con=getOption("TSconnection"), ...) standardGeneric("TSdoc"))
 
@@ -258,10 +267,12 @@ TSdocSQL <-  function(x=NULL, con=getOption("TSconnection"),
 	            "  FROM Meta ", setWhere(con, x, 
 		        realVintage(con, vintage, x),
 		        realPanel(con,panel)), ";", sep=""))[[1]]
-	    # r should already be char, but odbc converts NA to logical
-	    if(is.null(r))  as(NA, "character") else as(r, "character")
+	    # odbc converts NA to logical, but other db may return char
+	    if(is.null(r) || is.na(r)|| ("NA" == r)) NA else r
 	    }
 
+######### TSlabel #########
+
 setGeneric("TSlabel<-", 
    def= function(x, value) standardGeneric("TSlabel<-"),
    useAsDefault=  function (x, value){
@@ -270,7 +281,6 @@ setGeneric("TSlabel<-",
     TSmeta(x) <- m
     x})
 
-
 setGeneric("TSlabel", 
    def= function(x, con=getOption("TSconnection"), ...) standardGeneric("TSlabel"))
 
@@ -314,19 +324,86 @@ TSlabelSQL <-  function(x=NULL, con=getOption("TSconnection"),
 	    #        "  FROM Meta ", setWhere(con, x, 
 	    #            realVintage(con, vintage, x),
 	    #	         realPanel(con,panel)), ";", sep=""))[[1]]
-	    ## r should already be char, but odbc converts NA to logical
-	    #if(is.null(r)) as(NA, "character") else as(r, "character")
-	    as(NA, "character")
+	    # odbc converts NA to logical, but other db may return char
+	    #if(is.null(r) || is.na(r)|| ("NA" == r)) NA else r
+	    NA
 	    }
 
-#setMethod("show", "TSmeta", function(object) {
-#    cat("serIDs: ", object@serIDs, " from dbname: ", object@dbname) 
-#    cat("(type: ", object@conType, ")\n") 
-#    cat("description: ", object@TSdescription, "\n") 
-#    cat("documentaion: ", object@TSdoc, "\n") 
-#    invisible(object)
-#    })
-#
+######### TSsource #########
+
+setGeneric("TSsource<-", 
+   def= function(x, value) standardGeneric("TSsource<-"),
+   useAsDefault=  function (x, value){
+    m <- TSmeta(x)
+    m@TSsource <- value
+    TSmeta(x) <- m
+    x})
+
+
+setGeneric("TSsource", 
+   def= function(x, con=getOption("TSconnection"), ...) standardGeneric("TSsource"))
+
+setMethod("TSsource",   signature(x="character", con="missing"),
+   definition= function(x, con=getOption("TSconnection"), ...){ 
+       if(is.null(con)) 
+          stop("con should be specified or set with options(TSconnection=con). See ?TSsource.") 
+       TSsource(x, con=con, ...)} )
+
+setMethod("TSsource",   signature(x="ANY", con="missing"),
+   definition=  function(x, con, ...) TSmeta(x)@TSsource)
+
+#  next is for case where there is no method for con  
+setMethod("TSsource",   signature(x="character", con="ANY"),
+   definition= function(x, con=getOption("TSconnection"), ...) {
+       if(is.null(con)) stop("NULL con is not allowed. See ?TSsource.")
+       else stop("con class ", class(con), 
+       " is not supported. (Check this is a TSdbi connection, not a raw DBI connection.)")} )
+
+#  Next two methods are for case where the user mistakenly specifies
+#     serIDS="whatever"  rather than x="whatever"
+#   ( A natural mistaken as this is the syntax for other functions.)
+setMethod("TSsource",   signature(x="missing", con="ANY"),
+   definition=  function(x, con, serIDs, ...) {
+        if(missing(serIDs)) stop("missing argument x, must be specified.")
+	else TSsource(x=serIDs, con=con, ...)
+	})
+
+setMethod("TSsource",   signature(x="missing", con="missing"),
+   definition=  function(x, serIDs, ...) {
+        if(missing(serIDs)) stop("missing argument x, must be specified.")
+	else TSsource(x=serIDs, ...)
+	})
+
+TSsourceSQL <-  function(x=NULL, con=getOption("TSconnection"), 
+       vintage=getOption("TSvintage"), panel=getOption("TSpanel"), 
+       lang=getOption("TSlang")) {
+            if(1 < length(x)) stop("One series only for TSsource")
+	    #  NOT YET
+            #r <- dbGetQuery(con, paste("SELECT source", lang, 
+	    #        "  FROM Meta ", setWhere(con, x, 
+	    #            realVintage(con, vintage, x),
+	    #	         realPanel(con,panel)), ";", sep=""))[[1]]
+	    # odbc converts NA to logical, but other db may return char
+	    #if(is.null(r) || is.na(r)|| ("NA" == r)) NA else r
+	    NA
+	    }
+
+setMethod("show", "TSmeta", function(object) {
+    cat("serIDs: ", object@serIDs,"\n") 
+    if(!all(is.na(object@TSsource))) cat("source: ", object@TSsource,"\n") 
+    cat(" from dbname ", object@dbname) 
+    cat(" using ", object@conType) 
+        if(!is.na(object@DateStamp)) cat(" on ",as.character(object@DateStamp)) 
+    cat("\n") 
+    if(!is.na(object@TSlabel))       
+            cat("label:        ", object@TSlabel, "\n") 
+    if(!is.na(object@TSdescription))
+            cat("description: ", object@TSdescription, "\n") 
+    if(!is.na(object@TSdoc)) 
+            cat("documentaion: ", object@TSdoc, "\n") 
+    invisible(NULL)
+    })
+
 #setMethod("print", "TSmeta", function(x, ...) {
 #    cat("serIDs: ", x@serIDs, " from dbname ", x@dbname) 
 #    cat("(type: ", x@conType, ")\n") 
@@ -383,7 +460,8 @@ setMethod("TSput",   signature(x="ANY", serIDs="character", con="ANY"),
    
 
 TSputSQL <- function(x, serIDs=seriesNames(x), con, Table=NULL,
-       TSdescription.=TSdescription(x), TSdoc.=TSdoc(x),  TSlabel.=TSlabel(x), 
+       TSdescription.=TSdescription(x), TSdoc.=TSdoc(x),  TSlabel.=TSlabel(x),
+       TSsource.=TSsource(x), 
        vintage=getOption("TSvintage"), panel=getOption("TSpanel")) {
 
   # so far I think this is generic to all SQL, but not extensively tested.
@@ -596,7 +674,8 @@ setMethod("TSget",   signature(serIDs="character", con="ANY"),
 # next is called by methods for various SQL dbs.
 TSgetSQL <- function(serIDs, con, TSrepresentation=getOption("TSrepresentation"),
        tf=NULL, start=tfstart(tf), end=tfend(tf),
-       names=NULL, TSdescription=FALSE, TSdoc=FALSE,  TSlabel=FALSE,
+       names=NULL, 
+       TSdescription=FALSE, TSdoc=FALSE, TSlabel=FALSE, TSsource=TRUE,
        vintage=getOption("TSvintage"), panel=getOption("TSpanel")) {
   # so far I think this is generic to all SQL.
   if(is.null(TSrepresentation)) TSrepresentation <- "default"
@@ -624,7 +703,7 @@ TSgetSQL <- function(serIDs, con, TSrepresentation=getOption("TSrepresentation")
       as.matrix(res)
       }
 
-  mat <- desc <- doc <- label <- rp <- NULL
+  mat <- desc <- doc <- label <- source <- rp <- NULL
   # if series are in "A", "Q", "M","S" use  ts otherwise zoo.
   for (i in seq(length(serIDs))) {
     where  <-  setWhere(con, serIDs[i], vintage[i], panel)
@@ -710,9 +789,10 @@ TSgetSQL <- function(serIDs, con, TSrepresentation=getOption("TSrepresentation")
 	      " looking for series ", serIDs[i],
               " could not find tbl ", tbl)  
     
-    if(TSdescription) desc <- c(desc, TSdescription(serIDs[i],con) ) # where?
-    if(TSdoc)         doc  <- c(doc,  TSdoc(serIDs[i],con) ) # where?
-    if(TSlabel)       label<- c(label,TSlabel(serIDs[i],con) ) # where?
+    if(TSdescription) desc <- c(desc,  TSdescription(serIDs[i],con) ) # where?
+    if(TSdoc)         doc  <- c(doc,   TSdoc(serIDs[i],con) ) # where?
+    if(TSlabel)       label<- c(label, TSlabel(serIDs[i],con) ) # where?
+    if(TSsource)     source<- c(source,TSsource(serIDs[i],con) ) # where?
 
     mat <- tbind(mat, r)
     } # where[j]
@@ -730,9 +810,10 @@ TSgetSQL <- function(serIDs, con, TSrepresentation=getOption("TSrepresentation")
   TSmeta(mat) <- new("TSmeta", serIDs=serIDs, dbname=con@dbname, 
       conType=class(con), hasVintages=con@hasVintages, hasPanels=con@hasPanels, 
       DateStamp=NA, # bug in 2.7.0 =Sys.time(), 
-      TSdescription=if(TSdescription) desc else  as(NA, "character"), 
-      TSdoc        =if(TSdoc)          doc else  as(NA, "character"),
-      TSlabel      =if(TSlabel)      label else  as(NA, "character"))
+      TSdescription=if(TSdescription) desc else  NA, 
+      TSdoc        =if(TSdoc)          doc else  NA,
+      TSlabel      =if(TSlabel)      label else  NA,
+      TSsource     =if(TSsource)    source else  NA)
   mat
   }
 
